<?php
/**
 * Class for handle ajax request
 */
if ( ! class_exists( 'IPGivay_Ajax' )) {

    class IPGivay_Ajax
    {
        /**
         * Take ajax action name
         *
         * @var string
         */
        protected $_action;
        
        /**
         * Take ajax action handlers
         *
         * @var array
         */
        protected $_handlers = array(
            'ipgivay_save'            => 'save',
            'ipgivay_delete'          => 'delete',
            'ipgivay_image_add'       => 'image_add',
            'ipgivay_image_delete'    => 'image_delete',
            
            'ipgivay_upload'          => 'upload',
            'ipgivay_clearimage'      => 'clear_image',
            'ipgivay_getoptin'        => 'get_optin',
            
            'ipgivay_save_email'      => 'save_email',
            'ipgivay_savestat'        => 'save_stat',
            'ipgivay_getchart'        => 'get_chart',
            'ipgivay_raffle'          => 'raffle',
            'ipgivay_send_prize'      => 'send_prize'
        );
        
        /**
         * Class constructor
         *
         * @return void
         */
        public function __construct()
        {
        }
        
        /**
         * Get action name
         *
         * @return string
         */
        public function getAction()
        {
            return $this->_action;
        }
        
        /**
         * Set action
         *
         * @param  string $action
         * @return IPShortcode_Ajax
         */
        public function setAction( $action )
        {
            $this->_action = sanitize_title( $action );
        }
        
        /**
         * Execute request
         *
         * @param  string $action
         * @return IPShortcode_Ajax
         */
        public function exec()
        {
            $data = array_merge( $_GET, $_POST );
            if ( empty( $data['doajax'] ) || empty( $data['ipgivay'] ) || empty( $data['action'] ) ) {
                return true;
            }
            $this->setAction( $data['action'] );
            
            $action   = $this->getAction();
            $handlers = (array)$this->_handlers;
            if ( array_key_exists( $action, $handlers )) {
                $handler = $handlers[$action];
                if ( method_exists( $this, $handler )) {
                    $this->$handler();
                }
            }
            return true;
        }
        
        /**
         * Save email
         *
         * @return void
         */
        public function save_email()
        {
            $giveid = 0;
            if ( ! empty( $_POST['giveid'] )) {
                $giveid = absint( $_POST['giveid'] );
            }
            $data = array();
            $data['mbr_join_date'] = date( 'c' );
            
            if ( ! empty( $_POST['email'] )) {
                $data['mbr_email'] = sanitize_email( $_POST['email'] );
            }
            if ( ! empty( $_POST['name'] )) {
                $data['mbr_name'] = apply_filters( 'single_post_title', $_POST['name'] );
            }
            $data['page_id'] = $giveid;
            $data['mbr_ip']  = $_SERVER['REMOTE_ADDR'];
            $data['mbr_key'] = md5( $_SERVER['REMOTE_ADDR'] . $data['mbr_email'] . time() );
            
            $model = new IPGivay_Model();
            if ( ! $give = get_post( $giveid ))
            {
                $json_data = array(
                    'status'    => '0',
                    'message'   => __( 'Give object not found.', 'ipgivay' )
                );
                echo json_encode( $json_data );
                die();
            }
            if ( $member = $model->get_member_byemail( $giveid, $data['mbr_email'] )) 
            {
                $json_data = array(
                    'status'    => '0',
                    'message'   => __( 'You ever try to get this prize.', 'ipgivay' )
                );
                echo json_encode( $json_data );
                die();
            }
            
            $result = $model->insert_member( $data );
            if ( $result ) 
            {
                if ( get_option( 'permalink_structure' ) )  {
                    $base = get_option( '_ipgivay_page_base_url', 'contest' );
                    $link = get_home_url() . "/$base/" . $give->post_name . '?get=1';
                } else {
                    $link = get_home_url() . '/?giveid=' . $give->ID . '&get=1';
                }
                
                //Save stat
                $model->save_stat( $give, 'subscriber' );
                if ( ! empty( $_POST['referral'] )) {
                    $model->save_stat( $give, 'referral' );
                } else {
                    $model->save_stat( $give, 'direct' );
                }
                
                //Save referral
                $model->save_referral( $result, $give );
                
                $json_data = array(
                    'status'    => '1',
                    'id'        => $result,
                    'giveid'    => $giveid,
                    'redirect'  => $link,
                    'message'   => __( 'Your personal data we have saved, and you have chance to get a prize.', 'ipgivay' ),
                    'share'     => $this->_get_share_buttons( $result, $give )
                );
            } else {
                $json_data = array(
                    'status'    => '0',
                    'message'   => __( 'You can not get the prize because of a system error, please try again letter.', 'ipgivay' )
                );
            }
            echo json_encode( $json_data );
            die();
        }
        
        /**
         * Get share button
         *
         * @param  int    $member_id
         * @param  object $give
         * @return string
         */
        protected function _get_share_buttons( $member_id, $give )
        {
            $model   = new IPGivay_Model();
            $setting = new IPGivay_Setting( $give->ID, '__ipgivay-setting' );
            $member  = $model->get_member( $member_id );
            $share   = '';
            if ( $member ) 
            {
                if ( get_option( 'permalink_structure' ) )  {
                    $base = get_option( '_ipgivay_page_base_url', 'contest' );
                    $link = get_home_url() . "/$base/" . $give->post_name . '?get=1&ref=' . $member->mbr_key;
                } else {
                    $link = get_home_url() . '/?giveid=' . $give->ID . '&get=1&ref=' . $member->mbr_key;
                }
                $link    = urlencode( $link );
                $body    = $setting->getSetting( 'share_email_content', '' );
                $body    = str_replace( '{link}', $link, $body );
                
                $title   = $give->post_title;
                $socials = array(
                    'facebook',
                    'twitter',
                    'google-plus',
                    'pinterest',
                    'linkedin',
                    'tumblr',
                    'stumbleupon',
                    'digg',
                    'email'
                );
                $actions =  array(
                    'facebook'      => 'http://www.facebook.com/share.php?u=' . $link,
                    'twitter'       => 'https://twitter.com/share?url=' . $link . '&text=' . $title,
                    'google-plus'   => 'https://plusone.google.com/_/+1/confirm?hl=en&url=' . $link,
                    'pinterest'     => 'http://pinterest.com/pin/create/button/?url=' . $link . '&media=&description=' . $title,
                    'linkedin'      => 'http://www.linkedin.com/shareArticle?mini=true&url=' . $link .'&title=' . $title,
                    'tumblr'        => 'http://www.tumblr.com/share?v=3&u=' . $link,
                    'stumbleupon'   => 'http://www.stumbleupon.com/submit?url=' . $link . '&title=' . $title,
                    'digg'          => 'http://digg.com/submit?url=' . $link . '&title=' . $title,
                    'email'         => 'mailto:?subject=' . $title . '&body=' . $body
                );
                foreach( $socials as $social ) 
                {
                    $itype = $setting->getSetting( 'social_icon', 'square_flat' );
                    if ( $setting->getSetting( "social_$social", 'on' ) == 'off' ) {
                        continue;
                    }
                    $src    = IPGIVAY_BASE_URL . 'img/icon/' . $itype . '/' . $social . '.png';
                    $shl    = $actions[$social];
                    $share .= '<li class="ipgivay-social ipgivay-social-' . $social . '">' .
                                '<a href="' . $shl . '">' .
                                    '<img src="' . $src . '" alt="' . $social . '">' .
                                '</a>' .
                            '</li>';
                }
            }
            return $share;
        }
        
        /**
         * Add/Save style
         *
         * @return void
         */
        public function save()
        {
            $isAdd = false;
            $error = true;
            if ( ! empty( $_GET['add'] )) {
                $isAdd = true;
            }
            if ( ! empty( $_POST['giveid'] )) {
                $giveId = absint( $_POST['giveid'] );
                if ( $give = get_post( $giveId ) ) {
                    $this->_update_give( $give );
                } else {
                    $giveId = 0;
                }
            } else {
                $giveId = $this->_add_give();
            }
            if ( $giveId )
            {
                $error = false;
                $model = new IPGivay_Model();
                $model->save_setting( $giveId );
            }
            if ( $error ) {
                $status  = 0;
                $giveId  = 0;
                if ( $isAdd ) {
                    $message = __( 'Failed to create a giveaway.', 'ipgivay' );
                } else {
                    $message = __( 'Failed to save giveaway.', 'ipgivay' );
                }
            } else {
                $status = 1;
                if ( $isAdd ) {
                    $message = __( 'Successfully create a giveaway.', 'ipgivay' );
                } else {
                    $message = __( 'Successfully to save giveaway.', 'ipgivay' );
                }
            }
            echo json_encode( array(
                'status'   => $status,
                'message'  => $message,
                'giveid'   => $giveId
            ) );
            die();
        }
        
        /**
         * Add new giveaway
         *
         * @return int
         */
        protected function _add_give()
        {
            global $user_ID;
            
            if ( empty( $_POST['title'] )) {
                $title = 'Unknown';
            } else {
                $title = apply_filters( 'single_post_title', $_POST['title'] );
            }
            if ( $_POST['status'] == 'true' ) {
                $status = 'publish';
            } else {
                $status = 'draft';
            }
            $content = '';
            if ( ! empty( $_POST['description'] ) ) {
                $content = $_POST['description'];
            }
            $dataPost = array(
                'post_status'  => $status, 
                'post_type'    => 'ipgivay-item', 
                'post_author'  => $user_ID,
                'post_title'   => $title,
                'post_content' => $content
            );
            if ( ! empty( $_POST['slug'] )) {
                $dataPost['post_name'] = apply_filters( 'pre_term_name', $_POST['slug'] );
            }
            $result = wp_insert_post( $dataPost );
            return $result;
        }
        
        /**
         * Update giveaway
         *
         * @param  object $give
         * @return boolean
         */
        protected function _update_give( $give )
        {
            $result   = 0;
            $dataPost = array();
            if ( ! empty( $_POST['title'] ) && $_POST['title'] != $give->post_title ) {
                $dataPost['post_title'] = apply_filters( 'single_post_title', $_POST['title'] );
            }
            if ( $_POST['status'] == 'true' ) {
                $status = 'publish';
            } else {
                $status = 'draft';
            }
            if ( $status != $give->post_status ) {
                $dataPost['post_status'] = $status;
            }
            if ( ! empty( $_POST['slug'] ) && $_POST['slug'] != $give->post_name ) {
                $dataPost['post_name'] = apply_filters( 'pre_term_name', $_POST['slug'] );
            }
            if ( ! empty( $_POST['description'] ) && $_POST['description'] != $give->post_content ) {
                $dataPost['post_content'] = $_POST['description'];
            }
            if ( ! empty( $dataPost )) {
                $dataPost['ID'] = $give->ID;
                $result = wp_update_post( $dataPost );
            }
            return $result;
        }
        
        /**
         * Processing ajax for delete giveaway
         *
         * @return void
         */
        public function delete()
        {
            $item = false;
            if ( ! empty( $_REQUEST['id'] )) {
                $id   = absint( $_REQUEST['id'] );
                $item = get_post( $id );
            }
            if ( ! $item ) {
                echo json_encode( array(
                    'status'   => 0,
                    'message'  => __( 'Error system', 'ipgivay' )
                ) );
                die();
            }
            $result = wp_delete_post( $item->ID, true );
            if ( $result ) {
                echo json_encode( array(
                    'status'   => 1,
                    'message'  => __( 'Successfully remove the giveaway', 'ipgivay' ),
                    'redirect' => admin_url() . 'admin.php?page=ipgivay&delete=1'
                ) );
            } else {
                echo json_encode( array(
                    'status'  => 0,
                    'message' => __( 'Failed to remove the giveaway', 'ipgivay' )
                ) );
            }
            die();
        }
        
        /**
         * Processing ajax for add image
         *
         * @return void
         */
        public function image_add()
        {
            if ( ! empty( $_FILES['async-upload'] )) {
            
                $filetype = wp_check_filetype_and_ext( $_FILES['async-upload']['tmp_name'], $_FILES['async-upload']['name'], false );
                if ( ! wp_match_mime_types( 'image', $filetype['type'] ) ) {
                    echo json_encode( array(
                        'status'   => 0,
                        'message'  => __( 'The uploaded file is not a valid image. Please try again.', 'ipgivay' )
                    ) );
                    die();
                }
                $attachment_id = media_handle_upload( 'async-upload', 0, array() );
                if ( is_wp_error( $attachment_id ) ) {
                    echo json_encode( array(
                        'status'   => 0,
                        'message'  => $attachment_id->get_error_message()
                    ) );
                    die();
                }
                if ( ! $attachment = wp_prepare_attachment_for_js( $attachment_id ) ) 
                {
                    echo json_encode( array(
                        'status'   => 0,
                        'message'  => __( 'Error upload image.', 'ipgivay' )
                    ) );
                    die();
                }
                
                echo json_encode( array(
                    'status'   => 1,
                    'id'       => $attachment['id'],
                    'content'  => $this->_get_item_img( $attachment ),
                    'message'  => __( 'Successfully add image', 'ipgivay' )
                ));
            }
            die();
        }
        
        /**
         * Processing ajax for deleting image
         *
         * @return void
         */
        public function image_delete()
        {
            $result = false;
            if ( ! empty( $_POST['id'] )) 
            {
                $id     = absint( $_POST['id'] );
                $attach = get_post( $id );
                if ( $attach ) {
                    $result = wp_delete_attachment( $id, true );
                }
            }
            if ( $result ) 
            {
                $images = '';
                if ( ! empty( $_POST['images'] )) {
                    $_imgs = trim( $_POST['images'], '|' );
                    $data  = explode( '|', $_imgs );
                    foreach( $data as $k => $v ) {
                        if ( $v == $id ) {
                            unset($data[$k]);
                        }
                    }
                    $images = implode( '|', $data );
                }
                echo json_encode( array(
                    'images'   => $images,
                    'status'   => 1,
                    'message'  => __( 'Successfully deleting image', 'ipgivay' )
                ));
            } else {
                echo json_encode( array(
                    'status'   => 0,
                    'message'  => __( 'Failed deleting image', 'ipgivay' )
                ));
            }
            die();
        }
        
        /**
         * Mendapatkan item image
         *
         * @return void
         */
        protected function _get_item_img( $attach )
        {
            $src = '';
            if ( ! empty( $attach['sizes']['medium']['url'] ) ) {
                $src = $attach['sizes']['medium']['url'];
            } if ( ! empty( $attach['sizes']['full']['url'] ) ) {
                $src = $attach['sizes']['full']['url'];
            }
            $result = '<div data-id="' . $attach['id'] . '" class="ipgivay-img-item ipadm-clear">' .
                            '<div class="ipgivay-img-thumb">' .
                                '<img src="' . $src . '">' .
                            '</div>' .
                            '<div class="ipgivay-img-delete">' .
                                '<a href="#"><i class="faip fa-times-circle"></i></a>' .
                            '</div>' .
                        '</div>';
            
            return $result;
        }
        
        /**
         * Processing ajax for upload image
         *
         * @return void
         */
        public function upload()
        {
            if ( ! empty( $_FILES['async-upload'] )) {
            
                $filetype = wp_check_filetype_and_ext( $_FILES['async-upload']['tmp_name'], $_FILES['async-upload']['name'], false );
                if ( ! wp_match_mime_types( 'image', $filetype['type'] ) ) {
                    echo json_encode( array(
                        'status'  => 0,
                        'data'    => array(
                            'message'  => __( 'The uploaded file is not a valid image. Please try again.', 'ipgivay' ),
                            'filename' => $_FILES['async-upload']['name'],
                        )
                    ) );
                    die();
                }
                $attachment_id = media_handle_upload( 'async-upload', 0, array() );
                if ( is_wp_error( $attachment_id ) ) {
                    echo json_encode( array(
                        'status' => 0,
                        'data'    => array(
                            'message'  => $attachment_id->get_error_message(),
                            'filename' => $_FILES['async-upload']['name'],
                        )
                    ) );
                    die();
                }
                if ( ! $attachment = wp_prepare_attachment_for_js( $attachment_id ) ) {
                    die();
                }
                
                echo json_encode( array(
                    'status'   => 1,
                    'data'     => $attachment
                ));
            }
            die();
        }
        
        /**
         * Processing ajax for clearing image
         *
         * @return void
         */
        public function clear_image()
        {
            $attach = false;
            if ( ! empty( $_REQUEST['attach'] )) {
                $attachid = absint( $_REQUEST['attach'] );
                $attach   = get_post( $attachid );
            }
            if ( ! $attach ) {
                echo json_encode( array(
                    'status'  => 0,
                    'data'    => array(
                        'message'  => __( 'Error system', 'ipgivay' )
                    )
                ) );
                die();
            }
            
            $delete = wp_delete_attachment( $attachid, true );
            if ( $delete ) {
                echo json_encode( array(
                    'status' => 1,
                    'data'   => array(
                        'message'  => __( 'Successfully clearing image', 'ipgivay' )
                    )
                ));
            
            } else {
                echo json_encode( array(
                    'status'  => 0,
                    'data'    => array(
                        'message'  => __( 'Error clearing image', 'ipgivay' )
                    )
                ) );
            }
            die();
        }
        
        /**
         * Get responder data
         *
         * @return void
         */
        public function get_optin()
        {
            if ( ! empty( $_POST['html'] ))
            {
                $html = str_replace( "'", '"', $_POST['html'] );
                $data = IPGivay_Form::parse( $html );
                echo json_encode( array(
                    'status'    => 1,
                    'data'      => $data
                ) );
                die();
            }
            echo json_encode( array(
                'status'  => 0
            ) );
            die();
        }
        
        /**
         * Save stat
         *
         * @return void
         */
        public function save_stat()
        {
            if ( ! empty( $_POST['giveid'] ) && ! empty( $_POST['type'] )) 
            {
                $giveid = absint( $_POST['giveid'] );
                $type   = apply_filters( 'pre_term_name', $_POST['type'] );
                if ( $give = get_post( $giveid ) ) {
                    $model =  new IPGivay_Model();
                    $model->save_stat( $give, $type );
                }
            }
            echo json_encode( array(
                'status'  => 1
            ) );
            die();
        }
        
        /**
         * Get chart
         *
         * @return void
         */
        public function get_chart()
        {
            $data = array_merge( $_POST, $_GET );
            if ( ! empty( $data['giveid'] ) && ! empty( $data['type'] )) 
            {
                $giveid    = absint( $data['giveid'] );
                $type      = apply_filters( 'pre_term_name', $data['type'] );
                $dateStart = false;
                $dateEnd   = false;
                if ( ! empty( $data['start'] ) && ! empty( $data['end'] )) {
                    $timeEnd   = strtotime( $data['end'] );
                    $timeStart = strtotime( $data['start'] );
                    $dateStart = gmdate("Y-m-d", $timeStart);
                    $dateEnd   = gmdate("Y-m-d", $timeEnd);
                }
                $model     = new IPGivay_Model();
                $chart     = array();
                $results   = $model->get_statistic_data( $giveid, $type, $dateStart, $dateEnd );
                
                if ( ! empty( $results )) {
                    foreach( $results as $data ) {
                        $timestamp = strtotime( $data->gdate );
                        $phpdate   = getdate($timestamp);
                        if ( $type == 'ctr' ) {
                            $value = 0;
                            if ( ! empty( $data->click ) && ! empty( $data->view )) {
                                $rate  = 100 * intval($data->click) / intval($data->view);
                                $rate  = round( $rate, 2 );
                                $value = $rate;
                            }
                        } else {
                            $value = (int)$data->value;
                        }
                        $chart[] = array(
                            'value'    => $value,
                            'year'     => $phpdate['year'],
                            'mon'      => $phpdate['mon'] - 1,
                            'day'      => $phpdate['mday'],
                        );
                    }
                }
                if ( $type == 'view' ) {
                    $label = __( 'Views', 'ipgivay' );
                } elseif ( $type == 'subscriber' ) {
                    $label = __( 'Subscriber', 'ipgivay' );
                } elseif ( $type == 'share' ) {
                    $label = __( 'Shares', 'ipgivay' );
                } elseif ( $type == 'direct' ) {
                    $label = __( 'Direct Subscriber', 'ipgivay' );
                } elseif ( $type == 'referral' ) {
                    $label = __( 'Referral Subscriber', 'ipgivay' );
                }
                echo json_encode( array(
                    'status'            => 1,
                    'labelDate'         => __( 'Date', 'ipgivay' ),
                    'labelValue'        => $label,
                    'data'              => $chart
                ) );
            } else {
            
                echo json_encode( array(
                    'status' => 0
                ) );
            }
            die();
        }
        
        /**
         * Raffle winners
         *
         * @return void
         */
        public function raffle()
        {
            $data = array_merge( $_POST, $_GET );
            $give = false;
            if ( ! empty( $data['giveid'] )) 
            {
                $give_id = absint( $data['giveid'] );
                $give = get_post( $give_id );
            }
            if ( ! $give )
            {
                $json_data = array(
                    'status'    => '0',
                    'message'   => __( 'Give object not found.', 'ipgivay' )
                );
                echo json_encode( $json_data );
                die();
            }
            $model   = new IPGivay_Model();
            $setting = new IPGivay_Setting( $give->ID, '__ipgivay-setting' );
            $members = $model->get_raffle_members( $give_id );
            if ( empty( $members ))
            {
                $json_data = array(
                    'status'    => '0',
                    'message'   => __( 'There is no member who joined in this contest.', 'ipgivay' )
                );
                echo json_encode( $json_data );
                die();
            }
            $count  = $setting->getSetting( 'prize_winners', 1 );
            $todraw = array();
            foreach( $members as $member ) {
                $refcount = $member->mbr_referral;
                if ( ! isset( $todraw[$refcount] )) {
                    $todraw[$refcount] = array();
                }
                $todraw[$refcount][] = $member;
            }
            $drawcount = 0;
            $winners   = array();
            foreach( $todraw as $winns )
            {
                if ( $drawcount >= $count ) {
                    break;
                }
                shuffle( $winns );
                foreach( $winns as $winner ) 
                {
                    if ( $drawcount >= $count ) {
                        break;
                    }
                    $winners[] = $winner;
                    $drawcount++;
                }
            }
            //Save winners
            $this->_save_winners( $give, $winners );
            
            $now = date('c');
            $setting->setSetting( 'last_ruffle', $now );
            
            $json_data = array(
                'status'    => '1',
                'date'      => mysql2date( 'd F Y, h:i', $now ),
                'message'   => __( 'Successfully draw contest winners.', 'ipgivay' ),
                'html'      => $this->_get_winners_list( $winners )
            );
            echo json_encode( $json_data );
            die();
        }
        
        /**
         * Send prize
         *
         * @return void
         */
        public function send_prize()
        {
            $data    = array_merge( $_POST, $_GET );
            
            if ( empty( $data['award_subject'] ) || empty( $data['award_message'] ))
            {
                $json_data = array(
                    'status'    => '0',
                    'message'   => __( 'Award messages content have not been determined.', 'ipgivay' )
                );
                echo json_encode( $json_data );
                die();
            }
            
            $give = false;
            if ( ! empty( $data['giveid'] )) 
            {
                $give_id = absint( $data['giveid'] );
                $give = get_post( $give_id );
            }
            if ( ! $give )
            {
                $json_data = array(
                    'status'    => '0',
                    'message'   => __( 'Give object not found.', 'ipgivay' )
                );
                echo json_encode( $json_data );
                die();
            }
            $model   = new IPGivay_Model();
            $setting = new IPGivay_Setting( $give->ID, '__ipgivay-setting' );
            $winners = $model->get_winners( $give->ID );
            if ( empty( $winners ))
            {
                $json_data = array(
                    'status'    => '0',
                    'message'   => __( 'There is no winners in this contest.', 'ipgivay' )
                );
                echo json_encode( $json_data );
                die();
            }
            
            //Send mail
            $headers = array('Content-Type: text/html; charset=UTF-8');
            $subject = stripslashes( $data['award_subject'] );
            $message = stripslashes( $data['award_message'] );
            
            foreach( $winners as $winner ) {
                if ( ! empty( $winner->mbr_email )){
                    @wp_mail( $winner->mbr_email, $subject, $message, $headers );
                }
            }
            
            $now = date('c');
            $setting->setSetting( 'award_subject', $data['award_subject'] );
            $setting->setSetting( 'award_message', $data['award_message'] );
            $setting->setSetting( 'last_send_prize', $now );
            
            $json_data = array(
                'status'    => '1',
                'date'      => mysql2date( 'd F Y, h:i', $now ),
                'message'   => __( 'Successfully send contest prize to all winners.', 'ipgivay' )
            );
            echo json_encode( $json_data );
            die();
        }
        
        /**
         * Get list winners
         *
         * @param  array $give
         * @param  array $winners
         * @return void
         */
        protected function _save_winners( $give, $winners )
        {
            $model = new IPGivay_Model();
            $model->clear_winners( $give->ID );
            
            $result = true;
            foreach( $winners as $winner )
            {
                $toinsert = array(
                    'page_id'   => $give->ID,
                    'mbr_id'    => $winner->mbr_id,
                    'winn_date' => date( 'c' )
                );
                $result = $model->insert_winner( $toinsert );
            }
            return $result;
        }
        
        /**
         * Get list winners
         *
         * @param  array $winners
         * @return void
         */
        protected function _get_winners_list( $winners )
        {
            $html = '<div class="ipgivay-user-table">' .
                        '<table>' .
                            '<thead>' .
                                '<tr>' .
                                    '<th class="ipgivay-th-no">' . __( 'No', 'ipgivay' ) . '</th>' .
                                    '<th class="ipgivay-th-date">' . __( 'Join Date', 'ipgivay' ) . '</th>' .
                                    '<th class="ipgivay-th-email">' . __( 'Email', 'ipgivay' ) . '</th>' .
                                    '<th class="ipgivay-th-name">' . __( 'Name', 'ipgivay' ) . '</th>' .
                                    '<th class="ipgivay-th-entries">' . __( 'Entries', 'ipgivay' ) . '</th>' .
                                '</tr>' .
                            '</thead>' .
                            '<tbody>';
            
            $no = 1;
            foreach( $winners as $user )
            {
                $html .= '<tr>' .
                            '<td>' . $no . '</td>' .
                            '<td>' . mysql2date( 'd M Y, h:i', $user->mbr_join_date ) . '</td>' .
                            '<td>' . $user->mbr_email . '</td>' .
                            '<td>' . $user->mbr_name . '</td>' .
                            '<td>' . $user->mbr_referral . '</td>' .
                        '</tr>';
                $no++;
            }
            $html .=        '</tbody>' .
                        '</table>' .
                    '</div>';
            
            return $html;
        }
    }
}