<?php
if ( ! class_exists( 'IPGivay_Db' )) {

    /**
     * Class for manage plugin database
     */
    class IPGivay_Db
    {
        /**
         * Take db version
         *
         * @var string
         */
        public $version = '1.0';
        
        /**
         * Class construct
         *
         * @return void
         */
        public function __construct()
        {
            global $ipgivay_db_version;
            $ipgivay_db_version = $this->version;
        }
        
        /**
         * Database query
         *
         * @return string
         */
        public function query_member_referral()
        {
            global $wpdb;
            
            $table_name = $wpdb->prefix . 'ipgivay_member_referral';
            $sql = "CREATE TABLE $table_name (
                        mref_id          int(11)             NOT NULL AUTO_INCREMENT,
                        page_id          int(11)             NOT NULL,
                        mbr_id           int(11)             NOT NULL,
                        mref_mbr_id      int(11)             NOT NULL,
                        mref_date        datetime  DEFAULT '0000-00-00 00:00:00' NOT NULL,
                    PRIMARY KEY mref_id (mref_id),
                    KEY page_id (page_id),
                    KEY mbr_id (mbr_id)
            );";
            
            return $sql;
        }
        
        /**
         * Database query
         *
         * @return string
         */
        public function query_member()
        {
            global $wpdb;
            
            $table_name = $wpdb->prefix . 'ipgivay_member';
            $sql = "CREATE TABLE $table_name (
                        mbr_id           int(11)             NOT NULL AUTO_INCREMENT,
                        page_id          int(11)             NOT NULL,
                        mbr_ip           varchar(64)         NOT NULL,
                        mbr_key          varchar(244)        NOT NULL,
                        mbr_join_date    datetime  DEFAULT '0000-00-00 00:00:00' NOT NULL,
                        mbr_email        varchar(244)        NOT NULL,
                        mbr_name         varchar(244)        NOT NULL,
                        mbr_referral     int(11)             NOT NULL,
                    PRIMARY KEY member_id (mbr_id),
                    KEY page_id (page_id)
            );";
            
            return $sql;
        }
        
        /**
         * Database query
         *
         * @return string
         */
        public function query_winner()
        {
            global $wpdb;
            
            $table_name = $wpdb->prefix . 'ipgivay_winner';
            $sql = "CREATE TABLE $table_name (
                        winn_id          int(11)             NOT NULL AUTO_INCREMENT,
                        page_id          int(11)             NOT NULL,
                        mbr_id           int(11)             NOT NULL,
                        winn_date        datetime  DEFAULT '0000-00-00 00:00:00' NOT NULL,
                    PRIMARY KEY winner_id (winn_id),
                    KEY page_id (page_id),
                    KEY member_id (mbr_id)
            );";
            
            return $sql;
        }
        
        /**
         * Database query
         *
         * @return string
         */
        public function query_stat()
        {
            global $wpdb;
            
            $table_name = $wpdb->prefix . 'ipgivay_stat';
            $sql = "CREATE TABLE $table_name (
                        stat_id          int(11)             NOT NULL AUTO_INCREMENT,
                        page_id          int(11)             NOT NULL,
                        stat_date        datetime  DEFAULT '0000-00-00 00:00:00' NOT NULL,
                        stat_view        int(11)             NOT NULL,
                        stat_subscriber  int(11)             NOT NULL,
                        stat_share       int(11)             NOT NULL,
                        stat_direct      int(11)             NOT NULL,
                        stat_referral    int(11)             NOT NULL,
                    PRIMARY KEY stat_id (stat_id),
                    KEY page_id (page_id)
            );";

            return $sql;
        }
        
        /**
         * Database query
         *
         * @return string
         */
        public function query()
        {
            $sql  = '';
            $sql .= $this->query_member();
            $sql .= $this->query_member_referral();
            $sql .= $this->query_stat();
            $sql .= $this->query_winner();
            return $sql;
        }
        
        /**
         * install
         *
         * @return void
         */
        public function install()
        {
            global $wpdb, $ipgivay_db_version;
            $installed_ver = get_option( "ipgivay_db_version" );
            
            if( ! $installed_ver || ( $installed_ver != $ipgivay_db_version ) ) {

                $sql = $this->query();

                require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
                dbDelta( $sql );

                update_option( "ipgivay_db_version", $ipgivay_db_version );
            }
        }
        
        /**
         * update
         *
         * @return void
         */
        public function update()
        {
            global $ipgivay_db_version;
            if ( get_site_option( 'ipgivay_db_version' ) != $ipgivay_db_version ) {
                $this->install();
            }
        }
    }

}
?>