<?php
/*
Plugin Name: WP EZ Viral Contest - Lite
Plugin URI: 
Description: Plugin that can be used to create a very useful viral contest page to collect email address from the users.
Version: 1.0
Author: InstaPRoduct
Author URI: http://insta-product.com/
*/

/*
    LICENSE: This source file to be part of a system developed by InstaPRoduct. 
    All copyrights use this file and the code in it owned by InstaPRoduct. 
    It is strictly forbidden to use, copy, modify or distribute this file and 
    the code in it without the permission of the developer.
*/

//Change this value for rebranding plugin
define( 'IPGIVAY_TITLE', 'WP EZ Viral Contest - Lite' );
define( 'IPGIVAY_FILE', __FILE__ );
define( 'IPGIVAY_PATH', plugin_dir_path( __FILE__ ) );
define( 'IPGIVAY_BASE_URL', plugin_dir_url( __FILE__ ) );

require_once( IPGIVAY_PATH . "includes/setting.php" );
require_once( IPGIVAY_PATH . "includes/model.php" );
require_once( IPGIVAY_PATH . "includes/db.php" );
require_once( IPGIVAY_PATH . "includes/ajax.php" );
require_once( IPGIVAY_PATH . "includes/admin.php" );
require_once( IPGIVAY_PATH . "includes/font.php" );
require_once( IPGIVAY_PATH . "includes/form.php" );
require_once( IPGIVAY_PATH . "includes/factory.php" );

/**
 * Main class
 */
if ( ! class_exists( 'IPGivay' )) {

    class IPGivay
    {
        /**
         * Setup Plugin
         */
        public static function setup()
        {
            if ( ! is_admin() ) {
                session_start();
            }
            $hook = basename( IPGIVAY_FILE, '.php' );
            
            //Register admin menu
            add_action( 'admin_menu', array( __CLASS__, 'add_admin_menu' ) );
            
            //Plugin init
            add_action( 'admin_init', array( __CLASS__, 'admin_init' ) );
            
            //Activate plugin
            register_activation_hook( __FILE__, array( __CLASS__, 'install' ) );
            
            //Hook for plugin loaded
            add_action( 'plugins_loaded', array( __CLASS__, 'update_db' ) );
             
            //Rewrite Handler
            add_action( 'init', array( __CLASS__, 'rewrite_flush' ) );
            add_action( 'pre_get_posts', array( __CLASS__, 'init' ) );
            
            //Render page
            add_action( 'template_redirect', array( __CLASS__, 'render' ) );
            
            //Add admin style & script
            add_action( 'admin_print_styles', array( __CLASS__, 'admin_style' ), 1000 );
            add_action( 'admin_print_scripts', array( __CLASS__, 'admin_global_script' ), 900 );
            add_action( 'admin_print_scripts-toplevel_page_ipgivay', array( __CLASS__, 'admin_script' ), 1000 );
            add_action( 'admin_print_scripts-'.$hook.'_page_ipgivay-add', array( __CLASS__, 'admin_script' ), 1000 );
            add_action( 'admin_print_scripts-'.$hook.'_page_ipgivay-stat', array( __CLASS__, 'admin_script' ), 1000 );
            
        }
        
        /**
         * Install plugin
         *
         * @return void
         */
        public static function install()
        {
            //Rewrite
            self::rewrite_init();
            flush_rewrite_rules();
            
            //Db
            $db = new IPGivay_Db();
            $db->install();
        }
        
        /**
         * Update database
         *
         * @return void
         */
        public static function update_db()
        {
            $db = new IPGivay_Db();
            $db->update();
        }

        /**
         * Init request
         *
         * @return void
         */
        public static function init( $query )
        {
            $q = $query->query;
            if ( isset( $q['ipgivay'] ) && isset( $q['givename'] )) {
                $query->query_vars['post_type'] = 'ipgivay-item';
                $query->query_vars['name']      = $q['givename'];
                $query->query_vars['pagename']  = $q['givename'];
                $query->query['pagename']       = $q['givename'];
            
            } elseif( isset( $q['leadid'] )) {
                $query->query_vars['post_type'] = 'ipgivay-item';
                $query->query_vars['p']         = absint( $q['leadid'] );
            }
        }
        
        /**
         * Init rewrite rule
         *
         * @return void
         */
        public static function rewrite_init()
        {
            $base = get_option( '_ipgivay_page_base_url', 'contest' );
            add_rewrite_rule(
                $base . '/([^/]+)/?$',
                'index.php?ipgivay=1&givename=$matches[1]',
                'top'
            );
            add_rewrite_rule(
                $base . '/([^/]+)/feed/(feed|rdf|rss|rss2|atom)/?$',
                'index.php?ipgivay=1&givename=$matches[1]&feed=$matches[2]',
                'top'
            );
            add_rewrite_rule(
                $base . '/([^/]+)/(feed|rdf|rss|rss2|atom)/?$',
                'index.php?ipgivay=1&givename=$matches[1]&feed=$matches[2]',
                'top'
            );
            add_rewrite_rule(
                $base . '/([^/]+)/page/?([0-9]{1,})/?$',
                'index.php?ipgivay=1&givename=$matches[1]&paged=$matches[2]',
                'top'
            );
            add_rewrite_tag( '%givename%', '([a-zA-Z0-9\_])');
            add_rewrite_tag( '%giveid%', '([0-9])');
            add_rewrite_tag( '%ipgivay%', '([1])');
        }
        
        /**
         * Flush rewrite rule
         *
         * @return void
         */
        public static function rewrite_flush()
        {
            $base = get_option( '_ipgivay_page_base_url', 'contest' );
            $old  = get_option( '_ipgivay_page_base_url_old', 'contest' );
            
            self::rewrite_init();
            
            if ( $base != $old ) {
                flush_rewrite_rules();
                update_option( '_ipgivay_page_base_url_old', $base );
            }
        }
        
        /**
         * Add administrator menu
         *
         * @return void
         */
        public static function add_admin_menu()
        {
            add_menu_page( 
                IPGIVAY_TITLE, 
                IPGIVAY_TITLE, 
                'ipgivay_manage', 
                'ipgivay',
                array( __CLASS__, 'admin' ), 
                plugin_dir_url( __FILE__ ) . 'img/menu_icon.png'
            );
            add_submenu_page( 
                'ipgivay',
                __( 'All Contests', 'ipgivay' ), 
                __( 'All Contests', 'ipgivay' ), 
                'ipgivay_manage', 
                'ipgivay',
                array( __CLASS__, 'admin' )
            );
            add_submenu_page( 
                'ipgivay',
                __( 'Add Contest', 'ipgivay' ), 
                __( 'Add Contest', 'ipgivay' ), 
                'ipgivay_manage', 
                'ipgivay-add',
                array( __CLASS__, 'admin_add' )
            );
        }
        
        /**
         * Admin Init
         *
         * @return void
         */
        public static function admin_init()
        {
            global $wp_roles;
            
            /** 
             * Register Roles
             */
            // Add capability for administrator
            $role = get_role( 'administrator' );
            $role->add_cap( 'ipgivay_manage' );
            
            $wp_roles->reinit();
            
            /**
             * Ajax handler
             **/
            $ajax = new IPGivay_Ajax();
            $ajax->exec();
        }
        
        /**
         * Content for administrator page
         *
         * @return void
         */
        public static function admin()
        {
            $display = new IPGivay_Admin();
            if ( ! empty( $_GET['action'] ) && $_GET['action'] == 'edit' ) {
                $display->edit();
            } else {
                $display->index();
            }
        }
        
        /**
         * Content for administrator add contest page
         *
         * @return void
         */
        public static function admin_add()
        {
            $display = new IPGivay_Admin();
            $display->add();
        }
        
        /**
         * Render page
         *
         * @return void
         */
        public static function render()
        {
            //Factory page
            $factory = new IPGivay_Factory();
            $factory->render();
        }

        /**
         * Print admin style
         *
         * @return void
         */
        public static function admin_style()
        {
            wp_deregister_style('font-awesome-410');
            wp_register_style('font-awesome-410', plugin_dir_url( __FILE__ ) . 'css/font-awesome-4.1.0/css/font-awesome.css', false, '4.0.1');
            wp_enqueue_style('font-awesome-410');
            
            wp_deregister_style('insta-admin-style');
            wp_register_style('insta-admin-style', plugin_dir_url( __FILE__ ) . 'css/ipadmin.css', false, null);
            wp_enqueue_style('insta-admin-style');
            
            wp_deregister_style('ipgivay-datepicker');
            wp_register_style('ipgivay-datepicker', plugin_dir_url( __FILE__ ) . 'css/datepicker.css', false, null);
            wp_enqueue_style('ipgivay-datepicker');
            
            wp_deregister_style('ipgivay-timepicker');
            wp_register_style('ipgivay-timepicker', plugin_dir_url( __FILE__ ) . 'css/bootstrap-timepicker.min.css', false, null);
            wp_enqueue_style('ipgivay-timepicker');
            
            wp_deregister_style('ipgivay-admin-style');
            wp_register_style('ipgivay-admin-style', plugin_dir_url( __FILE__ ) . 'css/admin.css', false, null);
            wp_enqueue_style('ipgivay-admin-style');
        }

        /**
         * Print Admin Script
         *
         * @return void
         */
        public static function admin_script($hook)
        {
            wp_deregister_script('insta-admin-script');
            wp_enqueue_script(
                'insta-admin-script',
                plugin_dir_url( __FILE__ ) . 'js/ipadmin.js',
                array( 'jquery' ),
                1,
                true
            );
            wp_deregister_script('ipgivay-datepicker-script');
            wp_enqueue_script(
                'ipgivay-datepicker-script',
                plugin_dir_url( __FILE__ ) . 'js/bootstrap-datepicker.js',
                array( 'jquery' ),
                1,
                true
            );
            wp_deregister_script('ipgivay-timepicker-script');
            wp_enqueue_script(
                'ipgivay-timepicker-script',
                plugin_dir_url( __FILE__ ) . 'js/bootstrap-timepicker.js',
                array( 'jquery' ),
                1,
                true
            );
            wp_deregister_script('ipgivay-admin-script');
            wp_enqueue_script(
                'ipgivay-admin-script',
                plugin_dir_url( __FILE__ ) . 'js/admin.js',
                array( 'jquery' ),
                1,
                true
            );
        }

        /**
         * Print Global Admin Script
         *
         * @return void
         */
        public static function admin_global_script($hook)
        {
            wp_deregister_script('ipgivay-admin-global-script');
            wp_enqueue_script(
                'ipgivay-admin-global-script',
                plugin_dir_url( __FILE__ ) . 'js/admin-global.js',
                array( 'jquery' ),
                1,
                true
            );
        }
        
    }

}
//Run plugin
IPGivay::setup();
?>