<?php
if ( ! class_exists( 'IPGivay_Factory' )) {

    /**
     * Class for create landing page
     */
    class IPGivay_Factory
    {
        /**
         * Class constructor
         *
         * @return void
         */
        public function __construct()
        {
        }
        
        /**
         * Render page content
         *
         * @return string
         */
        public function render()
        {
            global $wp_query, $user_ID;
            if ( ( isset( $wp_query->query['ipgivay'] ) && isset( $wp_query->query['givename'] ) ) || 
                 isset( $wp_query->query['giveid'] )
            ){
                $give = $wp_query->post;
                if ( empty( $give ) ) {
                    return false;
                }
                $setting = new IPGivay_Setting( $give->ID, '__ipgivay-setting' );
                if ( $setting->getSetting( 'status', 'true' ) == 'false' ) {
                    return false;
                }
                if ( empty( $user_ID ) ) {
                    if ( $setting->getSetting( 'member_restricted', 'false' ) == 'true' ) {
                        return false;
                    }
                }
                
                $this->setting = $setting;
                $this->give    = $give;
                
                $file  = 'index.php';
                $theme = $setting->getSetting( 'template', 1 );
                $path  = IPGIVAY_PATH . 'themes/theme-' . $theme . '/' . $file;
                
                $this->theme      = $theme;
                $this->masterpath = IPGIVAY_PATH . 'themes/master/';
                $this->basepath   = IPGIVAY_PATH . 'themes/theme-' . $theme . '/';
                $this->baseurl    = plugins_url( 'themes/theme-' . $theme . '/', IPGIVAY_FILE  );
                $this->pluginurl  = plugins_url( '/', IPGIVAY_FILE  );
                $this->masterurl  = plugins_url( 'themes/master/', IPGIVAY_FILE  );
                
                /**
                 * Set global variable
                 */
                $this->url          = $this->_get_url();
                $this->bg           = $this->_get_bg();
                $this->google_font  = $this->_get_google_font();
                $this->root_class   = $this->_get_root_class();
                $this->button_class = $this->_get_button_class();
                $this->button_attr  = $this->_get_button_attr();
                $this->font_title   = $this->_get_font( 'title' );
                $this->font_text    = $this->_get_font();
                $this->logo         = $this->_get_logo();
                
                $this->is_start   = $this->_is_start();
                $this->is_finish  = $this->_is_finish();
                $this->is_submit  = $this->_is_submit();
                $this->is_already = $this->_is_already();
                $this->give_end   = $this->_get_date_end();
                $this->give_award = $this->_get_date_awarded();
                $this->give_start = $this->_get_date_start();
                $this->countdown  = $this->_get_countdown_time();
                $this->images     = $this->_get_images();
                
                $cd_start = false;
                if ( ! $this->is_start ) {
                    $cd_start = $this->_get_countdown_start();
                }
                $this->countdown_start  = $cd_start;
                
                $this->_set_content_data();
                
                if ( file_exists( $path )) {
                    include( $path );
                }
                
                //Save stat
                $model = new IPGivay_Model();
                $model->save_stat( $give, 'view' );
                
                exit();
            }
        }
        
        /**
         * Get page url
         *
         * @return string
         */
        protected function _get_url()
        {
            $url = '';
            if ( get_option( 'permalink_structure' ) )  {
                $base = get_option( '_ipgivay_page_base_url', 'contest' );
                $url  = get_home_url() . "/$base/" . $give->post_name . '?get=1';
            } else {
                $url = get_home_url() . '/?giveid=' . $give->ID . '&get=1';
            }
            return $url;
        }
        
        /**
         * Get background image
         *
         * @return string
         */
        protected function _get_bg()
        {
            $bg = false;
            if ( $img = $this->setting->getSetting( 'background_image' )) 
            {
                $src  = '';
                $data = wp_get_attachment_image_src( $img, 'full' );
                if ( ! empty( $data[0] )) {
                    $src = $data[0];
                }
                $repeat = $this->setting->getSetting( 'background_repeat', 'no-repeat' );
                $bg = ' style="background: url(' . $src . ') ' . $repeat . ';"';
            }
            return $bg;
        }
        
        /**
         * Get google font style
         *
         * @return string
         */
        protected function _get_google_font()
        {
            $fams    = array();
            $fonts   = array( 'roboto' );
            $fonts[] = $this->setting->getSetting( 'title_font_family' );
            $fonts[] = $this->setting->getSetting( 'text_font_family' );
            $fonts   = array_unique( $fonts );
            
            $links   = IPGivay_Font::getLinks();
            foreach( $fonts as $font ) 
            {
                if ( array_key_exists( $font, $links )) {
                    $fams[] = $links[$font];
                }
            }
            $family  = implode( '|', $fams );
            $fontUrl = 'http://fonts.googleapis.com/css?family=' . $family;
            $style   = '<link id="ipgivay-font" href="' . $fontUrl . '" rel="stylesheet" type="text/css">';
            return $style;
        }
        
        /**
         * Get root style
         *
         * @return string
         */
        protected function _get_root_class()
        {
            $sett      = $this->setting;
            $classes   = array();
            $classes[] = 'theme-' . $sett->getSetting( 'color_theme', 'grey' );
            $classes[] = 'font-' . $sett->getSetting( 'font_size', 'medium' );
            $classes[] = 'animation-' . $sett->getSetting( 'animation', 'none' );
            $classes[] = 'sound-' . $sett->getSetting( 'sound', '0' );
            $classes[] = 'transparency-' . $sett->getSetting( 'transparency', '0' );
            
            $class = implode( ' ', $classes );
            return $class;
        }
        
        /**
         * Get button style
         *
         * @return string
         */
        protected function _get_button_class()
        {
            $sett      = $this->setting;
            $classes   = array( 'btn' );
            $classes[] = 'btn-' . $sett->getSetting( 'button_theme', 'flat' );
            $classes[] = 'btn-' . $sett->getSetting( 'button_color', 'clean' );
            $classes[] = 'btn-' . $sett->getSetting( 'button_size', 'md' );
            
            $class = implode( ' ', $classes );
            return $class;
        }
        
        /**
         * Get button attr
         *
         * @return string
         */
        protected function _get_button_attr()
        {
            $sett  = $this->setting;
            $attr .= ' class="' . $this->_get_button_class() . '"';
            $attr .= $this->_get_font();
            return $attr;
        }
        
        /**
         * Get font style
         *
         * @param  string $type
         * @return string
         */
        protected function _get_font( $type = 'text' )
        {
            $font = '';
            if ( $value = $this->setting->getSetting( $type . '_font_family' )) 
            {
                $family = IPGivay_Font::getFamily( $value );
                $font   = ' style="font-family:' . $family . '"';
            }
            return $font;
        }
        
        /**
         * Get logo
         *
         * @return string
         */
        protected function _get_logo()
        {
            $logo = false;
            if ( $logo_img = $this->setting->getSetting( 'logo' )) 
            {
                $data = wp_get_attachment_image_src( $logo_img, 'full' );
                if ( ! empty( $data[0] )) {
                    $logo = $data[0];
                }
            }
            return $logo;
        }
        
        /**
         * Set content data
         *
         * @return void
         */
        protected function _set_content_data()
        {
            $st   = $this->setting;
            $this->gift = new stdClass();
            $this->gift->thumb   = $this->_get_content_thumb( 'prize_image' );
            $this->gift->brand   = stripslashes( $st->getSetting( 'prize_brand' ));
            $this->gift->title   = stripslashes( $st->getSetting( 'prize_name' ));
            $this->gift->content = stripslashes( $st->getSetting( 'prize_description' ));
            $this->gift->value   = stripslashes( $st->getSetting( 'prize_value' ));
            $this->gift->winners = stripslashes( $st->getSetting( 'prize_winners' ));
        }
        
        /**
         * Get content thumb
         *
         * @param  string $slug
         * @return false|string
         */
        protected function _get_content_thumb( $slug )
        {
            $thumb = false;
            if ( $img = $this->setting->getSetting( $slug )) 
            {
                $data = wp_get_attachment_image_src( $img, 'full' );
                if ( ! empty( $data[0] )) {
                    $thumb = $data[0];
                }
            }
            return $thumb;
        }
        
        /**
         * Is giveaway start?
         *
         * @return false
         */
        protected function _is_start()
        {
            $start  = false;
            $now    = time();
            $dstart = $this->setting->getSetting( 'start_date' );
            $tstart = $this->setting->getSetting( 'start_time' );
            if ( ! empty( $dstart ) && ! empty( $tstart )) {
                $date   = $dstart . ' ' . $tstart . ':00';
                $time   = $this->_create_date( $date, 'U' );
                if ( $now > $time ) {
                    $start = true;
                }
            }
            return $start;
        }
        
        /**
         * Is giveaway finish?
         *
         * @return false
         */
        protected function _is_finish()
        {
            $finish = false;
            $now    = time();
            $dend   = $this->setting->getSetting( 'end_date' );
            $tend   = $this->setting->getSetting( 'end_time' );
            if ( ! empty( $dend ) && ! empty( $tend )) {
                $date   = $dend . ' ' . $tend . ':00';
                $time   = $this->_create_date( $date, 'U' );
                if ( $now > $time ) {
                    $finish = true;
                }
            }
            return $finish;
        }
        
        /**
         * Is submitted
         *
         * @return false
         */
        protected function _is_submit()
        {
            $submit = false;
            if ( $this->give ) {
                $key = 'ipgivay_get_' . $this->give->ID;
                if ( isset( $_COOKIE[$key] )) {
                    $submit = true;
                }
            }
            return $submit;
        }
        
        /**
         * Is already
         *
         * @return false
         */
        protected function _is_already()
        {
            $already = false;
            $ip      = $_SERVER['REMOTE_ADDR'];
            $model   = new IPGivay_Model();
            if ( $model->get_member_byip( $this->give->ID, $ip ) ) {
                $already = true;
            }
            return $already;
        }
        
        /**
         * Get end date
         *
         * @return false
         */
        protected function _get_date_start()
        {
            $fdate  = get_option( 'date_format' );
            $ftime  = get_option( 'time_format' );
            $format = $fdate . ' ' . $ftime;
            $date   = date( 'F d, Y h:i' );
            if ( $startdate = $this->setting->getSetting( 'start_date' ) ) {
                if ( $starttime = $this->setting->getSetting( 'start_time' ) ) {
                    $startdate .= ' ' . $starttime . ':00';
                } else {
                    $startdate .= ' 00:00:00';
                }
                if ( $_date = $this->_create_date( $startdate, $format ) ) {
                    $date = $_date;
                }
            }
            return $date;
        }
        
        /**
         * Get end date
         *
         * @return false
         */
        protected function _get_date_end()
        {
            $fdate  = get_option( 'date_format' );
            $ftime  = get_option( 'time_format' );
            $format = $fdate . ' ' . $ftime;
            $date   = date( 'F d, Y h:i' );
            if ( $enddate = $this->setting->getSetting( 'end_date' ) ) {
                if ( $endtime = $this->setting->getSetting( 'end_time' ) ) {
                    $enddate .= ' ' . $endtime . ':00';
                } else {
                    $enddate .= ' 00:00:00';
                }
                if ( $_date = $this->_create_date( $enddate, $format ) ) {
                    $date = $_date;
                }
            }
            return $date;
        }
        
        /**
         * Get awarded date
         *
         * @return false
         */
        protected function _get_date_awarded()
        {
            $fdate  = get_option( 'date_format' );
            $ftime  = get_option( 'time_format' );
            $format = $fdate . ' ' . $ftime;
            $date   = date( 'F d, Y h:i' );
            if ( $awddate = $this->setting->getSetting( 'award_date' ) ) {
                if ( $awdtime = $this->setting->getSetting( 'award_time' ) ) {
                    $awddate .= ' ' . $awdtime . ':00';
                } else {
                    $awddate .= ' 00:00:00';
                }
                if ( $_date = $this->_create_date( $awddate, $format ) ) {
                    $date = $_date;
                }
            }
            return $date;
        }
        
        /**
         * Create date
         *
         * @return false
         */
        protected function _create_date( $date, $format )
        {
            try {
                $date = new DateTime( $date );
            } catch (Exception $e) {
                return false;
            }
            return $date->format( $format );
        }
        
        /**
         * Get countdown time
         *
         * @return string
         */
        protected function _get_countdown_time()
        {
            $time = '';
            $now  = time();
            $dend   = $this->setting->getSetting( 'end_date' );
            $tend   = $this->setting->getSetting( 'end_time' );
            if ( ! empty( $dend ) && ! empty( $tend )) {
                $date = $dend . ' ' . $tend . ':00';
                $time = str_replace( '-', '/', $date );
            }
            return $time;
        }
        
        /**
         * Get countdown start time
         *
         * @return string
         */
        protected function _get_countdown_start()
        {
            $time   = '';
            $dstart = $this->setting->getSetting( 'start_date' );
            $tstart = $this->setting->getSetting( 'start_time' );
            if ( ! empty( $dstart ) && ! empty( $tstart )) {
                $date = $dstart . ' ' . $tstart . ':00';
                $time = str_replace( '-', '/', $date );
            }
            return $time;
        }
        
        /**
         * Get images slider
         *
         * @return array
         */
        protected function _get_images()
        {
            $images = array();
            $active = $this->setting->getSetting( 'image_activate', 'false' );
            if ( $active == 'true' ) {
                if ( $imgid = $this->setting->getSetting( 'images' ) ) 
                {
                    $imgid  = trim( $imgid, '|' );
                    $imgids = explode( '|', $imgid );
                    foreach( $imgids as $id ) {
                        if ( ! empty( $id )) {
                            $url = wp_get_attachment_url( $id );
                            if ( $url ) {
                                $images[$id] = $url;
                            }
                        }
                    }
                }
            }
            return $images;
        }
    }
}