<?php
if ( ! class_exists( 'IPGivay_Model' )) {

    /**
     * Class model
     */
    class IPGivay_Model
    {
        /**
         * Save setting giveaway
         *
         * @param  int $giveId
         * @return void
         */
        public function save_setting( $giveId )
        {
            $giveId  = absint( $giveId );
            $data    = $this->_get_setting_data();
            $setting = new IPGivay_Setting( $giveId, '__ipgivay-setting' );
            $result  = $setting->setSettings( $data );
            return $result;
        }
        
        /**
         * Get setting data
         *
         * @return array
         */
        protected function _get_setting_data()
        {
            $setting = array();
            $data    = array_merge( $_POST, $_GET );
            
            /*
             * General
             */
            if ( isset( $data['status'] )) {
                $setting['status'] = apply_filters( 'pre_term_name', $data['status'] );
            }
            if ( isset( $data['type'] )) {
                $setting['type'] = apply_filters( 'pre_term_name', $data['type'] );
            }
            if ( isset( $data['start_date'] )) {
                $setting['start_date'] = preg_replace( '@[^a-zA-Z0-9\/\-\:]@', '', $data['start_date'] );
            }
            if ( isset( $data['start_time'] )) {
                $setting['start_time'] = preg_replace( '@[^a-zA-Z0-9\/\-\:]@', '', $data['start_time'] );
            }
            if ( isset( $data['end_date'] )) {
                $setting['end_date'] = preg_replace( '@[^a-zA-Z0-9\/\-\:]@', '', $data['end_date'] );
            }
            if ( isset( $data['end_time'] )) {
                $setting['end_time'] = preg_replace( '@[^a-zA-Z0-9\/\-\:]@', '', $data['end_time'] );
            }
            if ( isset( $data['award_date'] )) {
                $setting['award_date'] = preg_replace( '@[^a-zA-Z0-9\/\-\:]@', '', $data['award_date'] );
            }
            if ( isset( $data['award_time'] )) {
                $setting['award_time'] = preg_replace( '@[^a-zA-Z0-9\/\-\:]@', '', $data['award_time'] );
            }
            if ( isset( $data['member_restricted'] )) {
                $setting['member_restricted'] = apply_filters( 'pre_term_name', $data['member_restricted'] );
            }
            
            /*
             * Content
             */
            /** Content Gift **/
            if ( ! empty( $data['prize_name'] )) {
                $setting['prize_name'] = apply_filters( 'single_post_title', $data['prize_name'] );
            }
            if ( ! empty( $data['prize_brand'] )) {
                $setting['prize_brand'] = apply_filters( 'single_post_title', $data['prize_brand'] );
            }
            if ( ! empty( $data['prize_description'] )) {
                $setting['prize_description'] = $data['prize_description'];
            }
            if ( ! empty( $data['prize_image'] )) {
                $setting['prize_image'] = absint( $data['prize_image'] );
            }
            if ( ! empty( $data['prize_value'] )) {
                $setting['prize_value'] = apply_filters( 'single_post_title', $data['prize_value'] );
            }
            if ( ! empty( $data['prize_winners'] )) {
                $setting['prize_winners'] = absint( $data['prize_winners'] );
            }
            
            /** Content general **/
            if ( ! empty( $data['rules'] )) {
                $setting['rules'] = $data['rules'];
            }
            if ( isset( $data['question_activate'] )) {
                $setting['question_activate'] = apply_filters( 'pre_term_name', $data['question_activate'] );
            }
            if ( ! empty( $data['question'] )) {
                $setting['question'] = $data['question'];
            }
            if ( ! empty( $data['right_answer'] )) {
                $setting['right_answer'] = apply_filters( 'single_post_title', $data['right_answer'] );
            }
            if ( ! empty( $data['wrong_answer_1'] )) {
                $setting['wrong_answer_1'] = apply_filters( 'single_post_title', $data['wrong_answer_1'] );
            }
            if ( ! empty( $data['wrong_answer_2'] )) {
                $setting['wrong_answer_2'] = apply_filters( 'single_post_title', $data['wrong_answer_2'] );
            }
            if ( ! empty( $data['wrong_answer_3'] )) {
                $setting['wrong_answer_3'] = apply_filters( 'single_post_title', $data['wrong_answer_3'] );
            }
            if ( ! empty( $data['wrong_answer_4'] )) {
                $setting['wrong_answer_4'] = apply_filters( 'single_post_title', $data['wrong_answer_4'] );
            }
            if ( ! empty( $data['question_button'] )) {
                $setting['question_button'] = apply_filters( 'single_post_title', $data['question_button'] );
            }
            
            if ( ! empty( $data['get_gift_title'] )) {
                $setting['get_gift_title'] = apply_filters( 'single_post_title', $data['get_gift_title'] );
            }
            if ( ! empty( $data['already_gift_title'] )) {
                $setting['already_gift_title'] = apply_filters( 'single_post_title', $data['already_gift_title'] );
            }
            if ( ! empty( $data['not_started_gift_title'] )) {
                $setting['not_started_gift_title'] = apply_filters( 'single_post_title', $data['not_started_gift_title'] );
            }
            if ( ! empty( $data['ends_gift_title'] )) {
                $setting['ends_gift_title'] = apply_filters( 'single_post_title', $data['ends_gift_title'] );
            }
            if ( ! empty( $data['step_question_title'] )) {
                $setting['step_question_title'] = apply_filters( 'single_post_title', $data['step_question_title'] );
            }
            if ( ! empty( $data['step_email_title'] )) {
                $setting['step_email_title'] = apply_filters( 'single_post_title', $data['step_email_title'] );
            }
            
            if ( ! empty( $data['notstart_info'] )) {
                $setting['notstart_info'] = $data['notstart_info'];
            }
            if ( ! empty( $data['finished_info'] )) {
                $setting['finished_info'] = $data['finished_info'];
            }
            if ( ! empty( $data['already_get_info'] )) {
                $setting['already_get_info'] = $data['already_get_info'];
            }
            if ( ! empty( $data['after_get_info'] )) {
                $setting['after_get_info'] = $data['after_get_info'];
            }
            if ( ! empty( $data['ermsg_email_notset'] )) {
                $setting['ermsg_email_notset'] = $data['ermsg_email_notset'];
            }
            if ( ! empty( $data['ermsg_answer_notset'] )) {
                $setting['ermsg_answer_notset'] = $data['ermsg_answer_notset'];
            }
            if ( ! empty( $data['ermsg_answer_wrong'] )) {
                $setting['ermsg_answer_wrong'] = $data['ermsg_answer_wrong'];
            }
            
            /*
             * Image
             */
            if ( isset( $data['image_activate'] )) {
                $setting['image_activate'] = apply_filters( 'pre_term_name', $data['image_activate'] );
            }
            if ( isset( $data['images'] )) {
                $setting['images'] = apply_filters( 'single_post_title', $data['images'] );
            }
            
            /*
             * Email
             */
            if ( isset( $data['autoresponder'] )) {
                $setting['autoresponder'] = apply_filters( 'pre_term_name', $data['autoresponder'] );
            }
            if ( isset( $data['form_label'] )) {
                $setting['form_label'] = apply_filters( 'pre_term_name', $data['form_label'] );
            }
            if ( isset( $data['field_email_label'] )) {
                $setting['field_email_label'] = apply_filters( 'single_post_title', $data['field_email_label'] );
            }
            if ( isset( $data['field_name'] )) {
                $setting['field_name'] = apply_filters( 'pre_term_name', $data['field_name'] );
            }
            if ( isset( $data['field_name_label'] )) {
                $setting['field_name_label'] = apply_filters( 'single_post_title', $data['field_name_label'] );
            }
            if ( isset( $data['email_button'] )) {
                $setting['email_button'] = apply_filters( 'single_post_title', $data['email_button'] );
            }
            
            if ( isset( $data['autoresponder_code'] )) {
                $setting['autoresponder_code'] = $data['autoresponder_code'];
            }
            if ( isset( $data['autoresponder_action'] )) {
                $setting['autoresponder_action'] = apply_filters( 'pre_link_url', $data['autoresponder_action'] );
            }
            if ( isset( $data['autoresponder_email'] )) {
                $setting['autoresponder_email'] = apply_filters( 'pre_term_name', $data['autoresponder_email'] );
            }
            if ( isset( $data['autoresponder_email_label'] )) {
                $setting['autoresponder_email_label'] = apply_filters( 'single_post_title', $data['autoresponder_email_label'] );
            }
            if ( isset( $data['autoresponder_name'] )) {
                $setting['autoresponder_name'] = apply_filters( 'pre_term_name', $data['autoresponder_name'] );
            }
            if ( isset( $data['autoresponder_name_on'] )) {
                $setting['autoresponder_name_on'] = apply_filters( 'pre_term_name', $data['autoresponder_name_on'] );
            }
            if ( isset( $data['autoresponder_name_label'] )) {
                $setting['autoresponder_name_label'] = apply_filters( 'single_post_title', $data['autoresponder_name_label'] );
            }
            
            /*
             * Design
             */
            if ( isset( $data['template'] )) {
                $setting['template'] = apply_filters( 'pre_term_name', $data['template'] );
            }
            if ( isset( $data['color_theme'] )) {
                $setting['color_theme'] = apply_filters( 'pre_term_name', $data['color_theme'] );
            }
            if ( isset( $data['font_size'] )) {
                $setting['font_size'] = apply_filters( 'pre_term_name', $data['font_size'] );
            }
            if ( isset( $data['title_font_family'] )) {
                $setting['title_font_family'] = apply_filters( 'pre_term_name', $data['title_font_family'] );
            }
            if ( isset( $data['text_font_family'] )) {
                $setting['text_font_family'] = apply_filters( 'pre_term_name', $data['text_font_family'] );
            }
            if ( isset( $data['button_style'] )) {
                $setting['button_style'] = apply_filters( 'pre_term_name', $data['button_style'] );
            }
            if ( isset( $data['button_color'] )) {
                $setting['button_color'] = apply_filters( 'pre_term_name', $data['button_color'] );
            }
            if ( isset( $data['button_size'] )) {
                $setting['button_size'] = apply_filters( 'pre_term_name', $data['button_size'] );
            }
            if ( isset( $data['logo'] )) {
                $setting['logo'] = absint( $data['logo'] );
            }
            if ( isset( $data['background_image'] )) {
                $setting['background_image'] = absint( $data['background_image'] );
            }
            if ( isset( $data['background_repeat'] )) {
                $setting['background_repeat'] = apply_filters( 'pre_term_name', $data['background_repeat'] );
            }
            if ( isset( $data['transparency'] )) {
                $setting['transparency'] = apply_filters( 'pre_term_name', $data['transparency'] );
            }
            
            /*
             * Social
             */
            if ( isset( $data['social_facebook'] )) {
                $setting['social_facebook'] = apply_filters( 'pre_term_name', $data['social_facebook'] );
            }
            if ( isset( $data['social_twitter'] )) {
                $setting['social_twitter'] = apply_filters( 'pre_term_name', $data['social_twitter'] );
            }
            if ( isset( $data['social_google-plus'] )) {
                $setting['social_google-plus'] = apply_filters( 'pre_term_name', $data['social_google-plus'] );
            }
            if ( isset( $data['social_linkedin'] )) {
                $setting['social_linkedin'] = apply_filters( 'pre_term_name', $data['social_linkedin'] );
            }
            if ( isset( $data['social_tumblr'] )) {
                $setting['social_tumblr'] = apply_filters( 'pre_term_name', $data['social_tumblr'] );
            }
            if ( isset( $data['social_pinterest'] )) {
                $setting['social_pinterest'] = apply_filters( 'pre_term_name', $data['social_pinterest'] );
            }
            if ( isset( $data['social_reddit'] )) {
                $setting['social_reddit'] = apply_filters( 'pre_term_name', $data['social_reddit'] );
            }
            if ( isset( $data['social_stumbleupon'] )) {
                $setting['social_stumbleupon'] = apply_filters( 'pre_term_name', $data['social_stumbleupon'] );
            }
            if ( isset( $data['social_show'] )) {
                $setting['social_show'] = apply_filters( 'pre_term_name', $data['social_show'] );
            }
            if ( isset( $data['social_icon'] )) {
                $setting['social_icon'] = apply_filters( 'pre_term_name', $data['social_icon'] );
            }
            if ( ! empty( $data['share_info'] )) {
                $setting['share_info'] = $data['share_info'];
            }
            if ( isset( $data['share_title'] )) {
                $setting['share_title'] = apply_filters( 'single_post_title', $data['share_title'] );
            }
            if ( ! empty( $data['share_email_content'] )) {
                $setting['share_email_content'] = $data['share_email_content'];
            }
            
            /*
             * Effect
             */
            if ( isset( $data['animation'] )) {
                $setting['animation'] = apply_filters( 'pre_term_name', $data['animation'] );
            }
            if ( isset( $data['sound'] )) {
                $setting['sound'] = absint( $data['sound'] );
            }
            
            return $setting;
        }
        
        /**
         * Get members
         *
         * @return array
         */
        public function get_members()
        {
            global $wpdb;
            $table1 = $wpdb->prefix . 'ipgivay_member';
            $table2 = $wpdb->prefix . 'posts';
            $sql   = "
                SELECT * FROM $table1
                JOIN $table2 ON $table1.page_id = $table2.ID
                ORDER BY mbr_referral DESC
            ";
            $results = $wpdb->get_results( $sql );
            return $results;
        }
        
        /**
         * Get give members
         *
         * @param  int $give_id
         * @return array
         */
        public function get_raffle_members( $give_id )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'ipgivay_member';
            $id    = absint( $give_id );
            $sql   = "
                SELECT * FROM $table
                WHERE page_id = '$id'
                ORDER BY mbr_referral DESC
            ";
            $results = $wpdb->get_results( $sql );
            return $results;
        }
        
        /**
         * Get member by email
         *
         * @param  int $id
         * @return array
         */
        public function get_member( $id )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipgivay_member';
            $id     = absint( $id );
            $email  = sanitize_email( $email );
            $sql    = "
                SELECT * FROM $table WHERE mbr_id = '$id'
            ";
            $results = $wpdb->get_row( $sql );
            return $results;
        }
        
        /**
         * Get member by email
         *
         * @param  int $id
         * @param  string $email
         * @return array
         */
        public function get_member_byemail( $id, $email )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipgivay_member';
            $id     = absint( $id );
            $email  = sanitize_email( $email );
            $sql    = "
                SELECT * FROM $table WHERE page_id = '$id' AND mbr_email = '$email'
            ";
            $results = $wpdb->get_row( $sql );
            return $results;
        }
        
        /**
         * Get member by ip address
         *
         * @param  int    $id
         * @param  string $ip
         * @return array
         */
        public function get_member_byip( $id, $ip )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipgivay_member';
            $id     = absint( $id );
            $sql    = "
                SELECT * FROM $table WHERE page_id = '$id' AND mbr_ip = '$ip'
            ";
            $results = $wpdb->get_row( $sql );
            return $results;
        }
        
        /**
         * Get member by key
         *
         * @param  int    $id
         * @param  string $key
         * @return array
         */
        public function get_member_bykey( $id, $key )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipgivay_member';
            $id     = absint( $id );
            $sql    = "
                SELECT * FROM $table WHERE page_id = '$id' AND mbr_key = '$key'
            ";
            $results = $wpdb->get_row( $sql );
            return $results;
        }
        
        /**
         * Insert member
         *
         * @param  array $data
         * @param  array $format
         * @return void
         */
        public function insert_member( $data, $format = array() )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipgivay_member';
            $data   = (array)$data;
            $result = $wpdb->insert( $table, $data, $format );
            if ( $result ) {
                return $wpdb->insert_id;
            }
            return false;
        }
        
        /**
         * Update member
         *
         * @param  int   $id
         * @param  array $data
         * @return void
         */
        public function update_member( $id, $data )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipgivay_member';
            $data   = (array)$data;
            $where  = array(
                'mbr_id' => absint( $id )
            );
            $result = $wpdb->update( $table, $data, $where );
            return $result;
        }
        
        /**
         * Delete member
         *
         * @param  int   $id
         * @return void
         */
        public function delete_member( $id )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipgivay_member';
            $where  = array(
                'mbr_id' => absint( $id )
            );
            
            $result = $wpdb->delete( $table, $where );
            return $result;
        }
        
        /**
         * Insert member referral
         *
         * @param  array $data
         * @param  array $format
         * @return void
         */
        public function insert_member_referral( $data, $format = array() )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipgivay_member_referral';
            $data   = (array)$data;
            $result = $wpdb->insert( $table, $data, $format );
            if ( $result ) {
                return $wpdb->insert_id;
            }
            return false;
        }
        
        /**
         * Update member referral
         *
         * @param  int   $id
         * @param  array $data
         * @return void
         */
        public function update_member_referral( $id, $data )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipgivay_member_referral';
            $data   = (array)$data;
            $where  = array(
                'mref_id' => absint( $id )
            );
            $result = $wpdb->update( $table, $data, $where );
            return $result;
        }
        
        /**
         * Delete member referral
         *
         * @param  int   $id
         * @return void
         */
        public function delete_member_referral( $id )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipgivay_member_referral';
            $where  = array(
                'mref_id' => absint( $id )
            );
            
            $result = $wpdb->delete( $table, $where );
            return $result;
        }
        
        /**
         * Get winners
         *
         * @param int $page_id
         * @return array
         */
        public function get_winners( $page_id )
        {
            global $wpdb;
            
            $table1 = $wpdb->prefix . 'ipgivay_winner';
            $table2 = $wpdb->prefix . 'ipgivay_member';
            $sql    = "
                SELECT *
                FROM `$table1`
                JOIN $table2 ON $table1.mbr_id = $table2.mbr_id
                WHERE $table2.page_id = $page_id
            ";
            $results = $wpdb->get_results( $sql );
            return $results;
        }
        
        /**
         * Get stat
         *
         * @param  int $winner_id
         * @return array
         */
        public function get_winner( $winner_id )
        {
            global $wpdb;
            
            $id    = absint( $winner_id );
            $table = $wpdb->prefix . 'ipgivay_winner';
            $sql   = "
                SELECT *
                FROM `$table` WHERE winn_id = %d
            ";
            $query  = $wpdb->prepare( $sql, $id );
            $result = $wpdb->get_row( $query );
            return $result;
        }
        
        /**
         * Insert winner
         *
         * @param  array $data
         * @param  array $format
         * @return void
         */
        public function insert_winner( $data, $format = array() )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipgivay_winner';
            $data   = (array)$data;
            $result = $wpdb->insert( $table, $data, $format );
            if ( $result ) {
                return $wpdb->insert_id;
            }
            return false;
        }
        
        /**
         * Update winner
         *
         * @param  int   $id
         * @param  array $data
         * @return void
         */
        public function update_winner( $id, $data )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipgivay_winner';
            $data   = (array)$data;
            $where  = array(
                'winn_id' => absint( $id )
            );
            $result = $wpdb->update( $table, $data, $where );
            return $result;
        }
        
        /**
         * Delete winner
         *
         * @param  int   $id
         * @return void
         */
        public function delete_winner( $id )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipgivay_winner';
            $where  = array(
                'winn_id' => absint( $id )
            );
            
            $result = $wpdb->delete( $table, $where );
            return $result;
        }
        
        /**
         * Clear winner
         *
         * @param  int   $give_id
         * @return void
         */
        public function clear_winners( $give_id )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'ipgivay_winner';
            $where  = array(
                'page_id' => absint( $give_id )
            );
            $result = $wpdb->delete( $table, $where );
            return $result;
        }
        
        /**
         * Get stats
         *
         * @return array
         */
        public function get_stats()
        {
            global $wpdb;
            
            $table = $wpdb->prefix . 'ipgivay_stat';
            $sql   = "
                SELECT *
                FROM `$table`
            ";
            $results = $wpdb->get_results( $sql );
            return $results;
        }
        
        /**
         * Get stat
         *
         * @param  int $stat_id
         * @return array
         */
        public function get_stat( $stat_id )
        {
            global $wpdb;
            
            $stat_id = absint( $stat_id );
            $table   = $wpdb->prefix . 'ipgivay_stat';
            $sql     = "
                SELECT *
                FROM `$table` WHERE stat_id = %d
            ";
            $query   = $wpdb->prepare( $sql, $stat_id );
            $results = $wpdb->get_row( $query );
            return $results;
        }
        
        /**
         * Get stat value
         *
         * @param  int $page_id
         * @return array
         */
        public function get_stat_value( $page_id, $type )
        {
            $count = 0;
            if ( $stat = $this->get_stat_bypage( $page_id )) {
                if ( isset( $stat->$type )) {
                    $count = (int)$stat->$type;
                }
            }
            return $count;
        }
        
        /**
         * Get stat by page
         *
         * @param  int $page_id
         * @return array
         */
        public function get_stat_bypage( $page_id )
        {
            global $wpdb;
            
            $pageId = absint( $pageId );
            $table  = $wpdb->prefix . 'ipgivay_stat';
            $sql    = "
                SELECT 
                    SUM( `stat_view` ) AS `view`,
                    SUM( `stat_share` ) AS `share`,
                    SUM( `stat_subscriber` ) AS `subscriber`,
                    SUM( `stat_direct` ) AS `direct`,
                    SUM( `stat_referral` ) AS `referral`,
                FROM `$table` WHERE page_id = %d
            ";
            $query   = $wpdb->prepare( $sql, $page_id );
            $results = $wpdb->get_row( $query );
            return $results;
        }
        
        /**
         * Get stat by page date
         *
         * @param  int    $page_id
         * @param  string $date
         * @return array
         */
        public function get_stat_bypage_date( $page_id, $date )
        {
            global $wpdb;
            
            $page_id = absint( $page_id );
            $table   = $wpdb->prefix . 'ipgivay_stat';
            $sql     = "
                SELECT *
                FROM `$table` 
                WHERE page_id = $page_id and stat_date = '$date'
            ";
            $results = $wpdb->get_row( $sql );
            return $results;
        }
        
        /**
         * Get statistic data
         *
         * @return array
         */
        public function get_statistic_data( $page_id = 0, $type = 'view', $start = null, $end = null )
        {
            global $wpdb;
            
            $page_id = absint( $page_id );
            $table   = $wpdb->prefix . 'ipgivay_stat';
            $query   = "
                SELECT 
                    DATE_FORMAT(stat_date, '%Y-%m-%d') AS gdate,";
            
            //Type value
            if ( $type == 'share' ) {
                $query .= 'SUM(stat_share) AS value';
            } elseif ( $type == 'subscriber' ) {
                $query .= 'SUM(stat_subscriber) AS value';
            } elseif ( $type == 'direct' ) {
                $query .= 'SUM(stat_direct) AS value';
            } elseif ( $type == 'referral' ) {
                $query .= 'SUM(stat_referral) AS value';
            } else {
                $query .= 'SUM(stat_view) AS value';
            }
            $query .= " FROM $table";
            
            if ( ! empty( $page_id )) {
                $query .= " WHERE page_id = $page_id";
            }
            if ( ! empty( $start ) && ! empty( $end )) 
            {
                if ( $start == $end ) {
                    $query .= " AND DATE_FORMAT( stat_date, '%Y-%m-%d' ) = '$start'";
                } else {
                    $query .= " AND stat_date BETWEEN '$start' AND '$end'";
                }
            }
            elseif ( ! empty( $start )) {
                $query .= " AND DATE_FORMAT( stat_date, '%Y-%m-%d' ) = '$start'";
            }
            elseif ( ! empty( $end )) {
                $query .= " AND DATE_FORMAT( stat_date, '%Y-%m-%d' ) = '$end'";
            }
            $query .= " GROUP BY gdate";
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Do save statistic
         *
         * @param object $give
         * @param string $type
         * @param int    $count
         * @return void
         */
        public function save_stat( $give, $type, $count = 1 )
        {
            $_types = array(
                'view',
                'share',
                'subscriber',
                'direct',
                'referral'
            );
            $key = 'stat_';
            if ( in_array( $type, $_types )) {
                $key .= $type;
            }
            $date = date( 'Y-m-d 00:00:00' );
            $data = array();
            if ( $stat = $this->get_stat_bypage_date( $give->ID, $date )) 
            {
                $ccount = 0;
                if ( isset( $stat->$key )) {
                    $ccount = $stat->$key;
                }
                $data[$key] = $ccount + $count;
                $this->_update_stat( $stat->stat_id, $data );
            }
            else {
                $data['page_id']   = $give->ID;
                $data['stat_date'] = $date;
                $data[$key]        = $count;
                $this->_insert_stat( $data );
            }
        }
        
        /**
         * Insert statistic data
         *
         * @param array $data
         * @return void
         */
        protected function _insert_stat( $data )
        {
            global $wpdb;
            
            $table  = $wpdb->prefix . 'ipgivay_stat';
            $data   = (array)$data;
            
            $result = $wpdb->insert( $table, $data );
            if ( $result ) {
                return $wpdb->insert_id;
            }
            return false;
        }
        
        /**
         * Update statistic data
         *
         * @param int   $id
         * @param array $data
         * @return void
         */
        protected function _update_stat( $id, $data )
        {
            global $wpdb;
            
            $table  = $wpdb->prefix . 'ipgivay_stat';
            $data   = (array)$data;
            $where  = array(
                'stat_id' => absint( $id )
            );
            $result = $wpdb->update( $table, $data, $where );
            return $result;
        }
        
        /**
         * Save referral data
         *
         * @param int   $member_id
         * @param array $give
         * @return boolean
         */
        public function save_referral( $member_id, $give )
        {
            if ( empty( $_POST['referral'] )){
                return true;
            }
            if ( ! $member = $this->get_member( $member_id )){
                return false;
            }
            $refkey = apply_filters( 'pre_term_name', $_POST['referral'] );
            if ( ! $referral = $this->get_member_bykey( $give->ID, $refkey )){
                return false;
            }
            
            //Save data referral
            $toinsert = array(
                'page_id'       => $give->ID,
                'mbr_id'        => $member_id,
                'mref_mbr_id'   => $referral->mbr_id,
                'mref_date'     => date('c')
            );
            $this->insert_member_referral( $toinsert );
            
            //Update referral count
            $refcount = (int)$referral->mbr_referral;
            $refcount++;
            $toupdate = array(
                'mbr_referral'  => $refcount
            );
            $this->update_member( $referral->mbr_id, $toupdate );
            
            return true;
        }
    }
}